﻿/*
 * Copyright (c) 2011-2014 NecoTech.org, All rights reserved.
 */

#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

#include "spi.h"
#include "mmc.h"
#include "diskio.h"
#include "pff.h"
#include "mp3.h"

#define BUF_LEN  512

/* MP3データバッファー */
BYTE buf[BUF_LEN];

void blink_bled()
{
	for (int i = 0; i < 5; i++)
	{
		PORTD |= LED; _delay_ms(10); PORTD &= ~LED; _delay_ms(10);
	}
}

void blink_rled(int ms)
{
	for (int i = 0; i < 5; i++)
	{
		PORTB |= _BV(PORTB1);
		_delay_ms(ms);
		PORTB &= ~_BV(PORTB1);
		_delay_ms(ms);
	}
}

void once_led()
{
	PORTB |= _BV(PORTB1);
	_delay_ms(50);
	PORTB &= ~_BV(PORTB1);
}

int main()
{
	DDRB = 0b00101111;
	DDRD = 0b00001101;
	DDRC = 0b00000000;

	PORTD |= LED;
	_delay_ms(100);
	PORTD = 0;

	spi_init_low();

	mp3_init();

	PORTB |= _BV(PORTB3);
	PORTB |= _BV(PORTB4);
	PORTB |= _BV(PORTB5);

	_delay_ms(300);

	int buttonbusy = 0;

	FATFS fs;

	while (1)
	{
		spi_init_low();
		int rs = pf_mount(&fs);

		if (rs == FR_NOT_READY)
		{
			blink_rled(50);
		}
		else if (rs == FR_DISK_ERR)
		{
			blink_rled(300);
		}
		else if (rs == FR_NO_FILESYSTEM)
		{
			blink_rled(1000);
		}

		if (FR_OK == rs)
		{
			DIR dir;
			rs = pf_opendir(&dir, "/");
			if (rs == FR_OK)
			{
				FILINFO fno;

				while (1)
				{
					rs = pf_readdir(&dir, &fno);
					if (rs != FR_OK || fno.fname[0] == 0) break;

					if (fno.fattrib & AM_DIR) continue;

					rs = pf_open(fno.fname);
					if (rs == FR_OK)
					{
						once_led();

						WORD readBytes = 0;

						short isForward = 0;
						wait_for_ready();
						int i = 0;

						UNSELECT_XCS;
						SELECT_XDCS;

						BYTE *pbuf;
						while (1)
						{
							rs = pf_read(buf, BUF_LEN, &readBytes);
							if (readBytes <= 0) break;

							pbuf = buf;
							for (i = 0; i < readBytes; i++)
							{
								SPDR = *pbuf++;
								while (!(SPSR & (1 << SPIF))) {}

								while (!(PIND & DREQ));
							}
						}
					}
				}
			}
		}

		pf_mount(0);
		_delay_ms(1000);

	}

	return 0;
}
